import React from "react";
import ReactMapGL from "react-map-gl";
import {
  Marker,
  NavigationControl, 
  FullscreenControl,
  ScaleControl,
  GeolocateControl
} from "react-map-gl";
import "./css/maps.css";
import mapboxToken from './config/mapboxToken.js';
import {calcDistance} from './helpers/distance';
import {longitude, latitude, mapWidth, mapHeight, markerIcon} from "./config/variables";
  
export default class MapGL extends React.Component{
  state = {
    viewport: {
      width: mapWidth,
      height: mapHeight,
      latitude: latitude,
      longitude: longitude,
      zoom: 15,
      bearing: 0,
      pitch: 0
    },
    path: [[latitude, longitude]],
    distance: 0,
  };
  
  addMarker = event => {
    const lng = event.lngLat[0]
    const lat = event.lngLat[1]
    console.log("Clicked at", lat, "/", lng)
    let pathCopy = this.state.path.slice();
    pathCopy.push([lat, lng]);
    this.setState({path: pathCopy})
    this.getDistance()
  }

  updateMarker = (idx, event) => {
    const lng = event.lngLat[0]
    const lat = event.lngLat[1]
    console.log(lat, lng, idx)
    let pathCopy = this.state.path.slice();
    pathCopy[idx] = [lat, lng]
    this.setState({path: pathCopy})
    this.getDistance()
  }

  getDistance = () => {
    let dst = calcDistance(this.state.path)
    this.setState({distance: dst})
  }

  clearPath = () => {
    this.setState({path: [], distance: 0})
  }

  render() {
    return (
      <div>
        <div className="map">
          <ReactMapGL
            {...this.state.viewport}
            onViewportChange={(viewport) => this.setState({viewport})}
            mapboxApiAccessToken={mapboxToken}
            mapStyle="mapbox://styles/mapbox/streets-v11"
            onClick={this.addMarker}
          >

          {this.state.path.map((pos, idx) =>
            <Marker
              longitude={pos[1]}
              latitude={pos[0]}
              captureClick={false}
              draggable={true}
              offsetTop={-20}
              offsetLeft={-8}
              key={idx}
              onDragEnd={(e) => this.updateMarker(idx, e)}
            >
              <svg height={20} viewBox="0 0 24 24" style={{fill: "#343a40", stroke: "none"}}>
                <path d={markerIcon} />
              </svg>
            </Marker>
          )}

            <div className="controllers">
              <GeolocateControl />
              <FullscreenControl />
              <NavigationControl />
              <ScaleControl />
              <small> {'\u00A9'} Mapbox {'\u00A9'} OpenStreetMap </small>
            </div>
          </ReactMapGL>
        </div>

        <div className="mapInfo">
          <button onClick={this.clearPath}>Clear path</button>
          <button onClick={this.getDistance}>Calculate Distance</button>
          <p>
            Distanz: {Math.round(this.state.distance*100)/100} km
          </p>
        </div>
      </div>
    );
  }
}